using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 포커스 탐색 방향
        /// </summary>
        private FocusNavigationDirection focusNavigationDirection;

        /// <summary>
        /// 포커스 예측 여부
        /// </summary>
        private bool focusPredicted = false;

        /// <summary>
        /// 예측 컨트롤
        /// </summary>
        private Control predictedControl;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded                              += Window_Loaded;
            this.predictButton.PreviewMouseDown += predictButton_PreviewMouseDown;
            this.predictButton.PreviewMouseUp   += predictButton_PreviewMouseUp;
            this.moveButton.Click               += moveButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keyboard.Focus(this.firstButton);
        }

        #endregion
        #region 라디오 버튼 체크시 처리하기 - radioButton_Checked(sender, e)

        /// <summary>
        /// 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void radioButton_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton radioButton = e.Source as RadioButton;

            if(radioButton != null)
            {
                this.focusNavigationDirection = (FocusNavigationDirection)Enum.Parse
                (
                    typeof(FocusNavigationDirection),
                    (string)radioButton.Tag
                );
            }
        }

        #endregion
        #region 포커스 예측하기 버튼 프리뷰 마우스 DOWN 처리하기 - predictButton_PreviewMouseDown(sender, e)

        /// <summary>
        /// 포커스 예측하기 버튼 프리뷰 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void predictButton_PreviewMouseDown(object sender, RoutedEventArgs e)
        {
            DependencyObject predictionElement = null;

            UIElement focusedElement = Keyboard.FocusedElement as UIElement;

            if(focusedElement != null)
            {
                if
                (
                    (this.focusNavigationDirection == FocusNavigationDirection.Up   ) ||
                    (this.focusNavigationDirection == FocusNavigationDirection.Down ) ||
                    (this.focusNavigationDirection == FocusNavigationDirection.Left ) ||
                    (this.focusNavigationDirection == FocusNavigationDirection.Right)
                )
                {
                    predictionElement = focusedElement.PredictFocus(this.focusNavigationDirection);
                   
                    Control control = predictionElement as Control;

                    if(control != null)
                    {
                        control.Foreground = Brushes.DarkBlue;

                        control.FontSize += 10;

                        control.FontWeight = FontWeights.ExtraBold;

                        this.focusPredicted = true;

                        this.predictedControl = control;
                    }
                }
            }
        }

        #endregion
        #region 포커스 예측하기 버튼 프리뷰 마우스 UP 처리하기 - predictButton_PreviewMouseUp(sender, e)

        /// <summary>
        /// 포커스 예측하기 버튼 프리뷰 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void predictButton_PreviewMouseUp(object sender, RoutedEventArgs e)
        {
            if(this.focusPredicted == true)
            {
                this.predictedControl.Foreground = Brushes.Black;

                this.predictedControl.FontSize -= 10;

                this.predictedControl.FontWeight = FontWeights.Normal;

                this.focusPredicted = false;
            }
        }

        #endregion
        #region 포커스 이동하기 버튼 클릭시 처리하기 - moveButton_Click(sender, e)

        /// <summary>
        /// 포커스 이동하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void moveButton_Click(object sender, RoutedEventArgs e)
        {
            FocusNavigationDirection focusNavigationDirection = this.focusNavigationDirection;
            
            TraversalRequest traversalRequest = new TraversalRequest(focusNavigationDirection);

            UIElement focusesElement = Keyboard.FocusedElement as UIElement;

            if(focusesElement != null)
            {
                focusesElement.MoveFocus(traversalRequest);
            }
        }

        #endregion
    }
}